/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.web.tomcat.tc4.authenticator;


import java.io.IOException;
import java.security.Principal;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.catalina.HttpRequest;
import org.apache.catalina.HttpResponse;
import org.apache.catalina.authenticator.Constants;
import org.apache.catalina.deploy.LoginConfig;
import org.apache.catalina.util.Base64;


/**
 * An <b>Authenticator</b> and <b>Valve</b> implementation of HTTP BASIC
 * Authentication, as outlined in RFC 2617:  "HTTP Authentication: Basic
 * and Digest Access Authentication."
 * <p>
 * Differs from the standard Tomcat version in that it associates the session
 * of any request with any single sign-on session that may exist.
 *
 * @see #authenticate
 * @see AuthenticatorBase#associate
 * @see SingleSignOn
 *
 * @author B Stansberry, based on work by Craig R. McClanahan
 * @version $Revision: 1.1.2.2 $ $Date: 2005/12/01 19:51:37 $ based on Tomcat Revision 1.12
 */

public class BasicAuthenticator
      extends AuthenticatorBase
{


   // ----------------------------------------------------- Instance Variables


   /**
    * The Base64 helper object for this class.
    */
   protected static final Base64 base64Helper = new Base64();


   /**
    * Descriptive information about this implementation.
    */
   protected static final String info =
         BasicAuthenticator.class.getName() + "/1.0";


   // ------------------------------------------------------------- Properties


   /**
    * Return descriptive information about this Valve implementation.
    */
   public String getInfo()
   {

      return (this.info);

   }


   // --------------------------------------------------------- Public Methods


   /**
    * Authenticate the user making this request, based on the specified
    * login configuration.  Return <code>true</code> if any specified
    * constraint has been satisfied, or <code>false</code> if we have
    * created a response challenge already.
    * <p>
    * Differs from the standard Tomcat version in that it associates the
    * session of any request with any single sign-on session that may exist.
    *
    * @see AuthenticatorBase#associate
    * @see SingleSignOn
    *
    * @param request   Request we are processing
    * @param response  Response we are creating
    * @param config    Login configuration describing how authentication
    *                  should be performed
    *
    * @exception IOException if an input/output error occurs
    */
   public boolean authenticate(HttpRequest request,
                               HttpResponse response,
                               LoginConfig config)
         throws IOException
   {

      HttpServletRequest hreq =
            (HttpServletRequest) request.getRequest();

      // Have we already authenticated someone?
      // Note: SingleSignOn does not call setUserPrincipal(),
      // so whoever did it would have to be some custom valve
      // in the pipeline
      Principal principal = hreq.getUserPrincipal();
      String ssoId = (String) request.getNote(Constants.REQ_SSOID_NOTE);
      if (principal != null)
      {
         if (debug >= 1)
            log("Found principal '" + principal.getName() + "'");
         // Associate the session with any existing SSO session
         if (ssoId != null)
            associate(ssoId, getSession(request, true));
         return true;
      }

      // Is there an SSO session against which we can try to reauthenticate?
      if (ssoId != null)
      {
         if (debug >= 1)
            log("SSO Id set");
         // Try to reauthenticate using data cached by SSO.  If this fails,
         // either the original SSO logon was of DIGEST or SSL (which
         // we can't reauthenticate ourselves because there is no
         // cached username and password), or the realm denied
         // the user's reauthentication for some reason.
         // In either case we have to prompt the user for a logon */
         if (reauthenticateFromSSO(ssoId, request))
            return true;
      }

      // Validate any credentials already included with this request
      String authorization = request.getAuthorization();
      String username = parseUsername(authorization);
      String password = parsePassword(authorization);
      principal = context.getRealm().authenticate(username, password);
      if (principal != null)
      {
         register(request, response, principal, Constants.BASIC_METHOD,
                  username, password);
         return (true);
      }

      // Send an "unauthorized" response and an appropriate challenge
      String realmName = config.getRealmName();
      if (realmName == null)
         realmName = hreq.getServerName() + ":" + hreq.getServerPort();
      //        if (debug >= 1)
      //            log("Challenging for realm '" + realmName + "'");
      HttpServletResponse hres =
            (HttpServletResponse) response.getResponse();
      hres.setHeader("WWW-Authenticate",
                     "Basic realm=\"" + realmName + "\"");
      hres.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
      //      hres.flushBuffer();
      return (false);

   }


   // ------------------------------------------------------ Protected Methods


   /**
    * Parse the username from the specified authorization credentials.
    * If none can be found, return <code>null</code>.
    *
    * @param authorization Authorization credentials from this request
    */
   protected String parseUsername(String authorization)
   {

      if (authorization == null)
         return (null);
      if (!authorization.toLowerCase().startsWith("basic "))
         return (null);
      authorization = authorization.substring(6).trim();

      // Decode and parse the authorization credentials
      String unencoded =
            new String(base64Helper.decode(authorization.getBytes()));
      int colon = unencoded.indexOf(':');
      if (colon < 0)
         return (null);
      String username = unencoded.substring(0, colon).trim();
      //        String password = unencoded.substring(colon + 1).trim();
      return (username);

   }


   /**
    * Parse the password from the specified authorization credentials.
    * If none can be found, return <code>null</code>.
    *
    * @param authorization Authorization credentials from this request
    */
   protected String parsePassword(String authorization)
   {

      if (authorization == null)
         return (null);
      if (!authorization.startsWith("Basic "))
         return (null);
      authorization = authorization.substring(6).trim();

      // Decode and parse the authorization credentials
      String unencoded =
            new String(base64Helper.decode(authorization.getBytes()));
      int colon = unencoded.indexOf(':');
      if (colon < 0)
         return (null);
      //        String username = unencoded.substring(0, colon).trim();
      String password = unencoded.substring(colon + 1).trim();
      return (password);

   }


}
